﻿#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include  "game.h"

typedef struct game_s* game;
typedef const struct game_s* cgame;

struct game_s {
  int nb_moves;
  int width;
  int height;
  int nb_pieces;
  int **board;
  piece* tab_pieces;
}game_s;


game new_game(int width, int height, int nb_pieces, piece *pieces) {

  //Allocationde la structure de la game.
  game new_game = (game)malloc(sizeof(struct game_s));
  if (new_game == NULL) {
    exit(EXIT_FAILURE);
  }


  //Allocation de la partie horizontal du tableau de la game (axe x).
  new_game->board = (int**)malloc(sizeof(int*)*width);
  if (new_game->board == NULL) {
    exit(EXIT_FAILURE);
  }

  //Allocation de la partie verticale du tableau de la game (axe y).
  for(int i = 0; i<width; ++i){
    new_game->board[i] = (int*)malloc(sizeof(int)*height);
    if (new_game->board[i] == NULL) {
      exit(EXIT_FAILURE);
    }
  }

  //Initialisation des variables de la structure de game.
  new_game->width = width;
  new_game->height = height;
  new_game->tab_pieces = pieces;
  new_game->nb_pieces = nb_pieces;
  new_game->nb_moves = 0;

  //init_game(new_game);

  return new_game;
}

//Init_game sert à mettre les valeurs dans le tableau de game.
void init_game(game g){
  //initialisation de toute les cases du tableau a -1.
  for (int i = 0; i < g->width; ++i) {
    for (int j = 0; j < g->height; ++j) {
      g->board[i][j] = -1;
    }
  }

  //Pour chaques pieces de g->tab_pieces la position de la piece est mise dans g->board.
  for (int y = 0; y < game_nb_pieces(g); ++y) {
    if (can_move_x(g->tab_pieces[y])) {
      for (int a = get_x(g->tab_pieces[y]); a < get_width(g->tab_pieces[y]) + get_x(g->tab_pieces[y]); ++a) {
        g->board[(g->width-1)-get_x(g->tab_pieces[y])][get_height(g->tab_pieces[y])] = y;
      }
    }
    if (can_move_y(g->tab_pieces[y])) {
      for (int a = get_y(g->tab_pieces[y]); a < get_height(g->tab_pieces[y]) + get_y(g->tab_pieces[y]); ++a) {
        g->board[(g->width-1)][g->height-1-get_y(g->tab_pieces[y])] = y;
      }
    }
  }
}

//Libère toutes les cases du tableau de game, tableau et la structure.
void delete_game(game g) {
  for(int i=0; i<g->width ; ++i){
    free(g->board[i]);
  }
  free(g->board);
  free(g);

}

// Créer une copie de la game.
void copy_game(cgame src, game dst) {
  dst->nb_moves = src->nb_moves;
  for (int i = 0; i < game_width(src); ++i) {
    for (int j = 0; j < game_height(src); ++j) {
      dst->board[i][j] = src->board[i][j];
    }
  }
}

//Vérifie le nombre de piece en fonction de leur identifiant.
int game_nb_pieces(cgame g) {
  return g->nb_pieces;
}

//Retourne la piece demandée sinon envoie un message d'erreur avec une piece initialise à 0.
cpiece game_piece(cgame g, int piece_num) {
  if (piece_num < game_nb_pieces(g) && piece_num >= 0) {
    return *(g->tab_pieces + piece_num );
  }
  else {
    piece p = NULL;
    fprintf(stderr,"Erreur du numero de piece!\n");
    return p;
  }
}

//Vérifie la fin de la partie.
bool game_over_hr(cgame g) {
  if (get_x(g->tab_pieces[0])==4 && get_y(g->tab_pieces[0])==3) {
    return true;
  }
  return false;
}

bool game_over_ar(cgame g) {
  if (get_x(g->tab_pieces[0])==1 && get_y(g->tab_pieces[0])==0) {
    return true;
  }
  return false;
}

//Initialisela nouvelle position des pieces dans g->tab_pieces puis dans g->board.
bool play_move(game g, int piece_num, dir d, int distance) {

  if (piece_num <= game_nb_pieces(g)) {
    if(can_move_x(game_piece(g, piece_num))) {
      int x = get_x(game_piece(g, piece_num));
      int y = get_y(game_piece(g, piece_num));
      //int direction = is_horizontal(game_piece(g, piece_num));
      bool move_x = can_move_x(game_piece(g, piece_num));
      bool move_y = can_move_y(game_piece(g, piece_num));
      //bool taille;
      int height = get_height(game_piece(g, piece_num));
      int width = get_width(game_piece(g, piece_num));

      //copy sert à vérifier si la nouvelle piece n'entre pas en collision avec celles à proximités.
      piece copy;

      /*if (get_width(game_piece(g, piece_num))==2) {
	taille = 1;
	}
	else {
	taille = 0;
	}*/

      if (d == LEFT) {
	if (x - distance >= 0) {
	  copy = new_piece(x - distance, y, width, height, move_x, move_y );
	  for (int i = 0; i < game_nb_pieces(g); i++) {
	    if (i != piece_num) {
    	      //Vérification des collisions.
	      if (intersect(copy, *(g->tab_pieces + i))) {
		fprintf(stderr,"Impossible collision avec %d\n",i);
		delete_piece(copy);
		return 1;
	      }
	    }
	  }
    	  //Libération d'allocation de copy et de l'ancienne version de la piece.
	  delete_piece(copy);
	  delete_piece(*(g->tab_pieces + piece_num ));
    	  //Allocation de la nouvelle version piece.
	  *(g->tab_pieces + piece_num) = new_piece(x - distance, y, width, height, move_x, move_y );
	  g->nb_moves += distance;
	  init_game(g);
	  return 0;
	}
	else {
	  fprintf(stderr,"Impossible sortie du tableau\n");
	  return 1;
	}
      }

      if(d == RIGHT) {
	if (x + distance <=(game_width(g)-get_width(game_piece(g, piece_num)))) {
	  copy = new_piece(x + distance, y, width, height, move_x, move_y);
	  for (int i = 0; i < game_nb_pieces(g); i++) {
	    if (i != piece_num) {
    	      //Vérification des collisions.
	      if (intersect(copy, *(g->tab_pieces + i))) {
                fprintf(stderr,"Impossible collision avec %d\n",i);
                delete_piece(copy);
                return 1;
              }
            }
          }
    	  //Libération d'allocation de copy et de l'ancienne version de la piece.
          delete_piece(copy);
          delete_piece(*(g->tab_pieces + piece_num ));
    	  //Allocation de la nouvelle version piece.
          *(g->tab_pieces + piece_num ) = new_piece(x + distance, y, width, height, move_x, move_y );
          g->nb_moves += distance;
          init_game(g);
          return 0;
        }
        else{
	  fprintf(stderr,"Impossible sortie du tableau\n");
	  return 1;
	}
      }
    }

    if(can_move_y(game_piece(g, piece_num))) {
      int x = get_x(game_piece(g, piece_num));
      int y = get_y(game_piece(g, piece_num));
      //int direction = is_horizontal(game_piece(g, piece_num));
      bool move_x = can_move_x(game_piece(g, piece_num));
      bool move_y = can_move_y(game_piece(g, piece_num));
      //bool taille;
      int height = get_height(game_piece(g, piece_num));
      int width = get_width(game_piece(g, piece_num));

      //copy sert à vérifier si la nouvelle piece n'entre pas en collision avec celles à proximités.
      piece copy;

      /*if (get_height(game_piece(g, piece_num))==2) {
    	taille =1;
	}

	else {
    	taille = 0;
	}*/

      if (d == UP) {
	if (y + distance <= (game_height(g)-get_height(game_piece(g, piece_num)))) {
	  copy = new_piece(x, y + distance, width, height, move_x, move_y);
	  for (int i = 0; i < game_nb_pieces(g); i++) {
	    if (i != piece_num) {
    	      //Vérification des collisions.
	      if (intersect(copy, *(g->tab_pieces + i))) {
		fprintf(stderr,"Impossible collision avec %d\n",i);
		delete_piece(copy);
		return 1;
	      }
	    }
	  }
    	  //Libération d'allocation de copy et de l'ancienne version de la piece.
	  delete_piece(copy);
	  delete_piece(*(g->tab_pieces + piece_num ));
    	  //Allocation de la nouvelle version piece.
	  *(g->tab_pieces + piece_num ) = new_piece(x, y + distance, width, height, move_x, move_y);
	  g->nb_moves += distance;
	  init_game(g);
	  return 0;
	}
	else {
	  fprintf(stderr,"Impossible sortie du tableau\n");
	  return 1;
	}
      }

      if(d == DOWN) {
	if (y - distance >= 0) {
	  copy = new_piece(x, y - distance, width, height, move_x, move_y);
	  for (int i = 0; i < game_nb_pieces(g); i++) {
	    if (i != piece_num) {	   
    	      //Vérification des collisions.
	      if (intersect(copy, *(g->tab_pieces + i))) {
		fprintf(stderr,"Impossible collision avec %d\n",i);
		delete_piece(copy);
		return 1;
	      }
	    }
	  }
    	  //Libération d'allocation de copy et de l'ancienne version de la piece.
	  delete_piece(copy);
	  delete_piece(*(g->tab_pieces + piece_num ));
    	  //Allocation de la nouvelle version piece.
	  *(g->tab_pieces + piece_num ) = new_piece(x, y - distance, width, height, move_x, move_y);
	  g->nb_moves += distance;
	  init_game(g);
	  return 0;
	}
	else {
	  fprintf(stderr,"Impossible sortie du tableau\n");
	  return 1;
	}
      }
    }
  }
  fprintf(stderr, "Mauvais numéro de piece ou mauvaise orientation.\n");
  assert(false);
  return 1;
}


//Retourne le nombre de moves.
int game_nb_moves(cgame g) {
  return g->nb_moves;
}

//Retourne la longueur de g->board.
int game_width(cgame g){
  return g->width;
}

//Retourne la largeur de g->board.
int game_height(cgame g){
  return g->height;
}

int game_square_piece (cgame g, int x, int y){
  return g->board[x][y];
}
